import subprocess
from subprocess import check_output
import os
import sys
import datetime
from datetime import datetime
from collections import defaultdict

import click


def execute_on_remote(command, remote):
    return check_output(["ssh", remote, command]).decode("utf-8")


def copy_from_remote(source, target, remote):
    # run() does not properly preserve quotes when passed as a list
    subprocess.run(
        " ".join(["scp", "-T", "-r", f"{remote}:{source}", target]),
        shell=True,
        stderr=sys.stderr,
        stdout=sys.stdout,
    )


def print_available_space(disk, remote):
    command = "df -h"
    output = execute_on_remote(command, remote).strip().splitlines()
    output = [row.split()[:5] for row in output]

    for disk_name, size, used, avail, use_percent in output:
        if disk_name == disk:
            return click.echo(
                f"{disk}: Available: {avail}, Used: {used}/{size}({use_percent})"
            )

    click.secho("Specified disk not found", fg="red")


def list_dir(folder, remote):
    return execute_on_remote(f"ls {folder}", remote).strip().split()


def get_bags(source, remote):
    command = f"du -h --max-depth=1 {source}"
    bags = execute_on_remote(command, remote)
    bags = bags.strip().splitlines()[:-1]
    bags = [row.split() for row in bags]

    folders = defaultdict(list)

    for size, path in bags:
        bag = os.path.basename(path)
        dt = datetime.fromisoformat(bag)
        date = dt.strftime("%Y-%m-%d")
        time = dt.strftime("%H:%M")
        folders[date].append({"time": time, "size": size})

    return folders


def list_bags(source, remote):
    bag_tree = get_bags(source, remote)

    folders = sorted(bag_tree.keys())
    n_folders = len(folders)
    n = 0

    output = f"{source}\n"

    for i, folder in enumerate(folders):
        last_folder = i == n_folders - 1
        prefix = "└──" if last_folder else "├──"
        output += f"{prefix} {datetime.fromisoformat(folder).strftime('%d/%m/%Y')}\n"

        bags = sorted(bag_tree[folder], key=lambda bag: bag["time"])
        n_bags = len(bags)
        for j, bag in enumerate(bags):
            last_bag = j == n_bags - 1
            folder_prefix = "   " if last_folder else "│  "
            bag_prefix = "└──" if last_bag else "├──"
            output += f"{folder_prefix} {bag_prefix} [{click.style(n, fg='yellow')}] {click.style(bag['time'], fg='cyan')} ({bag['size']})\n"
            n += 1

    click.echo(output)


def delete_until(source, until, remote):
    bags = list_dir(source, remote)

    if bags[:until]:
        command = "rm -r"
        for bag in bags[:until]:
            command += f" {os.path.join(source, bag)}"
        execute_on_remote(command, remote)


def delete_keep(source, keep, remote):
    bags = list_dir(source, remote)

    if bags[:-keep]:
        command = "rm -r"
        for bag in bags[:-keep]:
            command += f" {os.path.join(source, bag)}"
        execute_on_remote(command, remote)


def copy_bag(source, target, n, remote, logs_only=False):
    bags = sorted(list_dir(source, remote))
    try:
        bag = bags[n]
    except IndexError:
        return click.secho("Selected bag does not exist", fg="red")

    path = os.path.join(source, bag)

    if logs_only:
        files = list_dir(path, remote)
        logs = [
            file
            for file in files
            if not file.endswith(".active") and not file.endswith(".bag")
        ]

        logs = [os.path.join(path, log) for log in logs]
        logs = " ".join(logs)
        logs = f'"{logs}"'

        target = os.path.join(target, bag)
        os.mkdir(target)
        copy_from_remote(logs, target, remote)

    else:
        copy_from_remote(os.path.join(source, bag), target, remote)



import json

import click

from lib import list_bags, print_available_space, delete_keep, delete_until, copy_bag
from config import get_config, save_config


@click.group()
@click.version_option()
def cli():
    """Zotac tools
    This package simplifies remote manipulation of rosbags on zotac.
    It lets you copy rosbags to your PC and delete old robags.
    It also provides a view of all rosbags grouped by date.
    """


@cli.command()
@click.argument("n", default=-2, type=int)
@click.argument("path", default=".", type=click.Path(exists=True))
@click.option("-l", "--logs-only", is_flag=True)
def copy(n, path, logs_only):
    config = get_config()
    copy_bag(config["log-directory"], path, n, config["remote"], logs_only=logs_only)


@cli.command()
@click.option("-u", "--until", type=int)
@click.option("-k", "--keep", type=int)
@click.option("-a", "--all", is_flag=True)
def delete(until, keep, all):
    config = get_config()
    if until:
        delete_until(config["log-directory"], until, config["remote"])
    elif keep:
        delete_keep(config["log-directory"], keep, config["remote"])
    elif all:
        delete_keep(config["log-directory"], 0, config["remote"])


@cli.command()
def status():
    config = get_config()
    print_available_space(config["disk-name"], config["remote"])


@cli.command()
def list():
    config = get_config()
    list_bags(config["log-directory"], config["remote"])


@cli.command("config")
@click.option("-s", "--show", is_flag=True)
@click.argument("args", nargs=-1)
def configure(show, args):
    config = get_config()

    if show:
        return click.echo(json.dumps(config, indent=2))

    d = dict(zip(args[::2], args[1::2]))
    for key, value in d.items():
        if key == "remote":
            config["remote"] = value
        elif key == "logdir" or key == "log-directory":
            config["log-directory"] = value
        elif key == "disk" or key == "disk-name":
            config["disk-name"] = value

    save_config(config)


if __name__ == "__main__":
    cli()



import pathlib
import os
import json

CONFIG_PATH = os.path.join(pathlib.Path(__file__).parent.resolve(), "config.json")


def get_config():
    with open(CONFIG_PATH) as f:
        return json.load(f)


def save_config(config):
    with open(CONFIG_PATH, "w") as f:
        json.dump(config, f, indent=2)
